@@ -63,6 +63,7 @@ from esphome.cpp_types import (  # noqa
     uint32,
     uint64,
     int32,
+    int64,
     const_char_ptr,
     NAN,
     esphome_ns,
@@ -75,10 +76,11 @@ from esphome.cpp_types import (  # noqa
     optional,
     arduino_json_ns,
     JsonObject,
-    JsonObjectRef,
-    JsonObjectConstRef,
+    JsonObjectConst,
     Controller,
     GPIOPin,
     InternalGPIOPin,
     gpio_Flags,
+    EntityCategory,
+    Parented,
 )
