@@ -31,18 +31,16 @@ uint16_t DallasTemperatureSensor::millis_to_wait_for_conversion() const {
 void DallasComponent::setup() {
   ESP_LOGCONFIG(TAG, "Setting up DallasComponent...");
 
-  yield();
+  pin_->setup();
+  one_wire_ = new ESPOneWire(pin_);  // NOLINT(cppcoreguidelines-owning-memory)
+
   std::vector<uint64_t> raw_sensors;
-  {
-    InterruptLock lock;
-    raw_sensors = this->one_wire_->search_vec();
-  }
+  raw_sensors = this->one_wire_->search_vec();
 
   for (auto &address : raw_sensors) {
-    std::string s = uint64_to_string(address);
     auto *address8 = reinterpret_cast<uint8_t *>(&address);
     if (crc8(address8, 7) != address8[7]) {
-      ESP_LOGW(TAG, "Dallas device 0x%s has invalid CRC.", s.c_str());
+      ESP_LOGW(TAG, "Dallas device 0x%s has invalid CRC.", format_hex(address).c_str());
       continue;
     }
     if (address8[0] != DALLAS_MODEL_DS18S20 && address8[0] != DALLAS_MODEL_DS1822 &&
@@ -54,7 +52,7 @@ void DallasComponent::setup() {
     this->found_sensors_.push_back(address);
   }
 
-  for (auto sensor : this->sensors_) {
+  for (auto *sensor : this->sensors_) {
     if (sensor->get_index().has_value()) {
       if (*sensor->get_index() >= this->found_sensors_.size()) {
         this->status_set_error();
@@ -70,7 +68,7 @@ void DallasComponent::setup() {
 }
 void DallasComponent::dump_config() {
   ESP_LOGCONFIG(TAG, "DallasComponent:");
-  LOG_PIN("  Pin: ", this->one_wire_->get_pin());
+  LOG_PIN("  Pin: ", this->pin_);
   LOG_UPDATE_INTERVAL(this);
 
   if (this->found_sensors_.empty()) {
@@ -78,8 +76,7 @@ void DallasComponent::dump_config() {
   } else {
     ESP_LOGD(TAG, "  Found sensors:");
     for (auto &address : this->found_sensors_) {
-      std::string s = uint64_to_string(address);
-      ESP_LOGD(TAG, "    0x%s", s.c_str());
+      ESP_LOGD(TAG, "    0x%s", format_hex(address).c_str());
     }
   }
 
@@ -97,29 +94,17 @@ void DallasComponent::dump_config() {
   }
 }
 
-DallasTemperatureSensor *DallasComponent::get_sensor_by_address(uint64_t address, uint8_t resolution) {
-  auto s = new DallasTemperatureSensor(address, resolution, this);
-  this->sensors_.push_back(s);
-  return s;
-}
-DallasTemperatureSensor *DallasComponent::get_sensor_by_index(uint8_t index, uint8_t resolution) {
-  auto s = this->get_sensor_by_address(0, resolution);
-  s->set_index(index);
-  return s;
-}
+void DallasComponent::register_sensor(DallasTemperatureSensor *sensor) { this->sensors_.push_back(sensor); }
 void DallasComponent::update() {
   this->status_clear_warning();
 
   bool result;
-  {
-    InterruptLock lock;
-    if (!this->one_wire_->reset()) {
-      result = false;
-    } else {
-      result = true;
-      this->one_wire_->skip();
-      this->one_wire_->write8(DALLAS_COMMAND_START_CONVERSION);
-    }
+  if (!this->one_wire_->reset()) {
+    result = false;
+  } else {
+    result = true;
+    this->one_wire_->skip();
+    this->one_wire_->write8(DALLAS_COMMAND_START_CONVERSION);
   }
 
   if (!result) {
@@ -130,11 +115,7 @@ void DallasComponent::update() {
 
   for (auto *sensor : this->sensors_) {
     this->set_timeout(sensor->get_address_name(), sensor->millis_to_wait_for_conversion(), [this, sensor] {
-      bool res;
-      {
-        InterruptLock lock;
-        res = sensor->read_scratch_pad();
-      }
+      bool res = sensor->read_scratch_pad();
 
       if (!res) {
         ESP_LOGW(TAG, "'%s' - Resetting bus for read failed!", sensor->get_name().c_str());
@@ -155,13 +136,7 @@ void DallasComponent::update() {
     });
   }
 }
-DallasComponent::DallasComponent(ESPOneWire *one_wire) : one_wire_(one_wire) {}
 
-DallasTemperatureSensor::DallasTemperatureSensor(uint64_t address, uint8_t resolution, DallasComponent *parent)
-    : parent_(parent) {
-  this->set_address(address);
-  this->set_resolution(resolution);
-}
 void DallasTemperatureSensor::set_address(uint64_t address) { this->address_ = address; }
 uint8_t DallasTemperatureSensor::get_resolution() const { return this->resolution_; }
 void DallasTemperatureSensor::set_resolution(uint8_t resolution) { this->resolution_ = resolution; }
@@ -170,13 +145,13 @@ void DallasTemperatureSensor::set_index(uint8_t index) { this->index_ = index; }
 uint8_t *DallasTemperatureSensor::get_address8() { return reinterpret_cast<uint8_t *>(&this->address_); }
 const std::string &DallasTemperatureSensor::get_address_name() {
   if (this->address_name_.empty()) {
-    this->address_name_ = std::string("0x") + uint64_to_string(this->address_);
+    this->address_name_ = std::string("0x") + format_hex(this->address_);
   }
 
   return this->address_name_;
 }
-bool ICACHE_RAM_ATTR DallasTemperatureSensor::read_scratch_pad() {
-  ESPOneWire *wire = this->parent_->one_wire_;
+bool IRAM_ATTR DallasTemperatureSensor::read_scratch_pad() {
+  auto *wire = this->parent_->one_wire_;
   if (!wire->reset()) {
     return false;
   }
@@ -190,11 +165,7 @@ bool ICACHE_RAM_ATTR DallasTemperatureSensor::read_scratch_pad() {
   return true;
 }
 bool DallasTemperatureSensor::setup_sensor() {
-  bool r;
-  {
-    InterruptLock lock;
-    r = this->read_scratch_pad();
-  }
+  bool r = this->read_scratch_pad();
 
   if (!r) {
     ESP_LOGE(TAG, "Reading scratchpad failed: reset");
@@ -228,21 +199,18 @@ bool DallasTemperatureSensor::setup_sensor() {
       break;
   }
 
-  ESPOneWire *wire = this->parent_->one_wire_;
-  {
-    InterruptLock lock;
-    if (wire->reset()) {
-      wire->select(this->address_);
-      wire->write8(DALLAS_COMMAND_WRITE_SCRATCH_PAD);
-      wire->write8(this->scratch_pad_[2]);  // high alarm temp
-      wire->write8(this->scratch_pad_[3]);  // low alarm temp
-      wire->write8(this->scratch_pad_[4]);  // resolution
-      wire->reset();
+  auto *wire = this->parent_->one_wire_;
+  if (wire->reset()) {
+    wire->select(this->address_);
+    wire->write8(DALLAS_COMMAND_WRITE_SCRATCH_PAD);
+    wire->write8(this->scratch_pad_[2]);  // high alarm temp
+    wire->write8(this->scratch_pad_[3]);  // low alarm temp
+    wire->write8(this->scratch_pad_[4]);  // resolution
+    wire->reset();
 
-      // write value to EEPROM
-      wire->select(this->address_);
-      wire->write8(0x48);
-    }
+    // write value to EEPROM
+    wire->select(this->address_);
+    wire->write8(0x48);
   }
 
   delay(20);  // allow it to finish operation
@@ -267,7 +235,7 @@ float DallasTemperatureSensor::get_temp_c() {
 
   return temp / 128.0f;
 }
-std::string DallasTemperatureSensor::unique_id() { return "dallas-" + uint64_to_string(this->address_); }
+std::string DallasTemperatureSensor::unique_id() { return "dallas-" + str_lower_case(format_hex(this->address_)); }
 
 }  // namespace dallas
 }  // namespace esphome
