@@ -4,12 +4,12 @@
 namespace esphome {
 namespace pulse_meter {
 
-static const char *TAG = "pulse_meter";
+static const char *const TAG = "pulse_meter";
 
 void PulseMeterSensor::setup() {
   this->pin_->setup();
   this->isr_pin_ = pin_->to_isr();
-  this->pin_->attach_interrupt(PulseMeterSensor::gpio_intr, this, CHANGE);
+  this->pin_->attach_interrupt(PulseMeterSensor::gpio_intr, this, gpio::INTERRUPT_ANY_EDGE);
 
   this->last_detected_edge_us_ = 0;
   this->last_valid_edge_us_ = 0;
@@ -23,7 +23,6 @@ void PulseMeterSensor::loop() {
   const uint32_t time_since_valid_edge_us = now - this->last_valid_edge_us_;
   if ((this->last_valid_edge_us_ != 0) && (time_since_valid_edge_us > this->timeout_us_)) {
     ESP_LOGD(TAG, "No pulse detected for %us, assuming 0 pulses/min", time_since_valid_edge_us / 1000000);
-    this->last_detected_edge_us_ = 0;
     this->last_valid_edge_us_ = 0;
     this->pulse_width_us_ = 0;
   }
@@ -36,7 +35,7 @@ void PulseMeterSensor::loop() {
       this->publish_state(0);
     } else {
       // Calculate pulses/min from the pulse width in ms
-      this->publish_state((60.0 * 1000.0) / pulse_width_ms);
+      this->publish_state((60.0f * 1000.0f) / pulse_width_ms);
     }
   }
 
@@ -48,6 +47,8 @@ void PulseMeterSensor::loop() {
   }
 }
 
+void PulseMeterSensor::set_total_pulses(uint32_t pulses) { this->total_pulses_ = pulses; }
+
 void PulseMeterSensor::dump_config() {
   LOG_SENSOR("", "Pulse Meter", this);
   LOG_PIN("  Pin: ", this->pin_);
@@ -55,29 +56,26 @@ void PulseMeterSensor::dump_config() {
   ESP_LOGCONFIG(TAG, "  Assuming 0 pulses/min after not receiving a pulse for %us", this->timeout_us_ / 1000000);
 }
 
-void ICACHE_RAM_ATTR PulseMeterSensor::gpio_intr(PulseMeterSensor *sensor) {
+void IRAM_ATTR PulseMeterSensor::gpio_intr(PulseMeterSensor *sensor) {
   // This is an interrupt handler - we can't call any virtual method from this method
 
   // Get the current time before we do anything else so the measurements are consistent
   const uint32_t now = micros();
 
   // We only look at rising edges
-  if (!sensor->isr_pin_->digital_read()) {
+  if (!sensor->isr_pin_.digital_read()) {
     return;
   }
 
-  // Ignore the first detected pulse (we need at least two pulses to measure the width)
-  if (sensor->last_detected_edge_us_ != 0) {
-    // Check to see if we should filter this edge out
-    if ((now - sensor->last_detected_edge_us_) >= sensor->filter_us_) {
-      // Don't measure the first valid pulse (we need at least two pulses to measure the width)
-      if (sensor->last_valid_edge_us_ != 0) {
-        sensor->pulse_width_us_ = (now - sensor->last_valid_edge_us_);
-      }
-
-      sensor->total_pulses_++;
-      sensor->last_valid_edge_us_ = now;
+  // Check to see if we should filter this edge out
+  if ((now - sensor->last_detected_edge_us_) >= sensor->filter_us_) {
+    // Don't measure the first valid pulse (we need at least two pulses to measure the width)
+    if (sensor->last_valid_edge_us_ != 0) {
+      sensor->pulse_width_us_ = (now - sensor->last_valid_edge_us_);
     }
+
+    sensor->total_pulses_++;
+    sensor->last_valid_edge_us_ = now;
   }
 
   sensor->last_detected_edge_us_ = now;
