@@ -1,6 +1,6 @@
 """Constants used by esphome."""
 
-__version__ = "2022.1.2"
+__version__ = "2022.2.2"
 
 ALLOWED_NAME_CHARS = "abcdefghijklmnopqrstuvwxyz0123456789-_"
 
@@ -62,6 +62,7 @@ CONF_BINARY_SENSORS = "binary_sensors"
 CONF_BINDKEY = "bindkey"
 CONF_BIRTH_MESSAGE = "birth_message"
 CONF_BIT_DEPTH = "bit_depth"
+CONF_BLOCK = "block"
 CONF_BLUE = "blue"
 CONF_BOARD = "board"
 CONF_BOARD_FLASH_MODE = "board_flash_mode"
@@ -106,6 +107,7 @@ CONF_COLOR_MODE = "color_mode"
 CONF_COLOR_TEMPERATURE = "color_temperature"
 CONF_COLORS = "colors"
 CONF_COMMAND = "command"
+CONF_COMMAND_RETAIN = "command_retain"
 CONF_COMMAND_TOPIC = "command_topic"
 CONF_COMMENT = "comment"
 CONF_COMMIT = "commit"
@@ -166,6 +168,7 @@ CONF_DIRECTION = "direction"
 CONF_DIRECTION_OUTPUT = "direction_output"
 CONF_DISABLED_BY_DEFAULT = "disabled_by_default"
 CONF_DISCOVERY = "discovery"
+CONF_DISCOVERY_OBJECT_ID_GENERATOR = "discovery_object_id_generator"
 CONF_DISCOVERY_PREFIX = "discovery_prefix"
 CONF_DISCOVERY_RETAIN = "discovery_retain"
 CONF_DISCOVERY_UNIQUE_ID_GENERATOR = "discovery_unique_id_generator"
@@ -187,6 +190,7 @@ CONF_ECO2 = "eco2"
 CONF_EFFECT = "effect"
 CONF_EFFECTS = "effects"
 CONF_ELSE = "else"
+CONF_ENABLE_IPV6 = "enable_ipv6"
 CONF_ENABLE_PIN = "enable_pin"
 CONF_ENABLE_TIME = "enable_time"
 CONF_ENERGY = "energy"
@@ -237,7 +241,9 @@ CONF_FORCE_UPDATE = "force_update"
 CONF_FORMALDEHYDE = "formaldehyde"
 CONF_FORMAT = "format"
 CONF_FORWARD_ACTIVE_ENERGY = "forward_active_energy"
+CONF_FRAGMENTATION = "fragmentation"
 CONF_FRAMEWORK = "framework"
+CONF_FREE = "free"
 CONF_FREQUENCY = "frequency"
 CONF_FROM = "from"
 CONF_FULL_SPECTRUM = "full_spectrum"
@@ -329,10 +335,12 @@ CONF_LINE_THICKNESS = "line_thickness"
 CONF_LINE_TYPE = "line_type"
 CONF_LOADED_INTEGRATIONS = "loaded_integrations"
 CONF_LOCAL = "local"
+CONF_LOCK_ACTION = "lock_action"
 CONF_LOG_TOPIC = "log_topic"
 CONF_LOGGER = "logger"
 CONF_LOGS = "logs"
 CONF_LONGITUDE = "longitude"
+CONF_LOOP_TIME = "loop_time"
 CONF_LOW = "low"
 CONF_LOW_VOLTAGE_REFERENCE = "low_voltage_reference"
 CONF_MAC_ADDRESS = "mac_address"
@@ -420,9 +428,11 @@ CONF_ON_ENROLLMENT_SCAN = "on_enrollment_scan"
 CONF_ON_FINGER_SCAN_MATCHED = "on_finger_scan_matched"
 CONF_ON_FINGER_SCAN_UNMATCHED = "on_finger_scan_unmatched"
 CONF_ON_JSON_MESSAGE = "on_json_message"
+CONF_ON_LOCK = "on_lock"
 CONF_ON_LOOP = "on_loop"
 CONF_ON_MESSAGE = "on_message"
 CONF_ON_MULTI_CLICK = "on_multi_click"
+CONF_ON_OPEN = "on_open"
 CONF_ON_PRESS = "on_press"
 CONF_ON_RAW_VALUE = "on_raw_value"
 CONF_ON_RELEASE = "on_release"
@@ -433,8 +443,10 @@ CONF_ON_TAG = "on_tag"
 CONF_ON_TAG_REMOVED = "on_tag_removed"
 CONF_ON_TIME = "on_time"
 CONF_ON_TIME_SYNC = "on_time_sync"
+CONF_ON_TOUCH = "on_touch"
 CONF_ON_TURN_OFF = "on_turn_off"
 CONF_ON_TURN_ON = "on_turn_on"
+CONF_ON_UNLOCK = "on_unlock"
 CONF_ON_VALUE = "on_value"
 CONF_ON_VALUE_RANGE = "on_value_range"
 CONF_ONE = "one"
@@ -702,6 +714,7 @@ CONF_UART_ID = "uart_id"
 CONF_UID = "uid"
 CONF_UNIQUE = "unique"
 CONF_UNIT_OF_MEASUREMENT = "unit_of_measurement"
+CONF_UNLOCK_ACTION = "unlock_action"
 CONF_UPDATE_INTERVAL = "update_interval"
 CONF_UPDATE_ON_BOOT = "update_on_boot"
 CONF_URL = "url"
@@ -804,6 +817,7 @@ ICON_WIFI = "mdi:wifi"
 
 UNIT_AMPERE = "A"
 UNIT_BECQUEREL_PER_CUBIC_METER = "Bq/m³"
+UNIT_BYTES = "B"
 UNIT_CELSIUS = "°C"
 UNIT_COUNT_DECILITRE = "/dL"
 UNIT_COUNTS_PER_CUBIC_METER = "#/m³"
@@ -832,6 +846,7 @@ UNIT_MICROMETER = "µm"
 UNIT_MICROSIEMENS_PER_CENTIMETER = "µS/cm"
 UNIT_MICROTESLA = "µT"
 UNIT_MILLIGRAMS_PER_CUBIC_METER = "mg/m³"
+UNIT_MILLISECOND = "ms"
 UNIT_MINUTE = "min"
 UNIT_OHM = "Ω"
 UNIT_PARTS_PER_BILLION = "ppb"
@@ -906,6 +921,9 @@ DEVICE_CLASS_VOLTAGE = "voltage"
 DEVICE_CLASS_UPDATE = "update"
 # device classes of button component
 DEVICE_CLASS_RESTART = "restart"
+# device classes of switch component
+DEVICE_CLASS_OUTLET = "outlet"
+DEVICE_CLASS_SWITCH = "switch"
 
 
 # state classes
